import datetime
import os

from sickchill import logger, settings, tv

from . import common
from .common import DOWNLOADED, Quality
from .name_parser.parser import InvalidNameException, InvalidShowException, NameParser

name_presets = ("%SN - %Sx%0E", "%SN - %Sx%0E - %EN", "%S.N.S%0SE%0E.%E.N", "%Sx%0E - %EN", "S%0SE%0E - %EN", "Season %0S/%S.N.S%0SE%0E.%Q.N-%RG")

name_anime_presets = name_presets

name_abd_presets = ("%SN - %A-D - %EN", "%S.N.%A.D.%E.N.%Q.N", "%Y/%0M/%S.N.%A.D.%E.N-%RG")

name_sports_presets = ("%SN - %A-D - %EN", "%S.N.%A.D.%E.N.%Q.N", "%Y/%0M/%S.N.%A.D.%E.N-%RG")


class TVShow(object):
    def __init__(self):
        self.name = "Show Name"
        self.genre = "Comedy"
        self.indexerid = 1
        self.air_by_date = 0
        self.sports = 0
        self.anime = 0
        self.scene = 0

    @property
    def is_anime(self):
        """
        Find out if show is anime
        :return: True if show is anime, False if not
        """
        return self.anime > 0

    @property
    def is_sports(self):
        """
        Find out if show is sports
        :return: True if show is sports, False if not
        """
        return self.sports > 0

    @property
    def is_scene(self):
        """
        Find out if show is scene numbering
        :return: True if show is scene numbering, False if not
        """
        return self.scene > 0


class TVEpisode(tv.TVEpisode):
    def __init__(self, season, episode, absolute_number, name):
        self.relatedEps = []
        self.name = name
        self.season = season
        self.episode = episode
        self.absolute_number = absolute_number
        self.scene_season = season
        self.scene_episode = episode
        self.scene_absolute_number = absolute_number
        self.airdate = datetime.date(2010, 3, 9)
        self.show = TVShow()
        self.status = Quality.compositeStatus(common.DOWNLOADED, common.Quality.SDTV)
        self.release_name = "Show.Name.S02E03.HDTV.XviD-SICKCHILL"
        self.is_proper = True


def check_force_season_folders(pattern=None, multi=None, anime_type=None):
    """
    Checks if the name can still be parsed if you strip off the folders to determine if we need to force season folders
    to be enabled or not.

    :return: true if season folders need to be forced on or false otherwise.
    """
    if pattern is None:
        pattern = settings.NAMING_PATTERN

    valid = not validate_name(pattern, None, anime_type, file_only=True)

    if multi is not None:
        valid = valid or not validate_name(pattern, multi, anime_type, file_only=True)

    return valid


def check_valid_naming(pattern=None, multi=None, anime_type=None):
    """
    Checks if the name is can be parsed back to its original form for both single and multi episodes.

    :return: true if the naming is valid, false if not.
    """
    if pattern is None:
        pattern = settings.NAMING_PATTERN

    logger.debug("Checking whether the pattern " + pattern + " is valid for a single episode")
    valid = validate_name(pattern, None, anime_type)

    if multi is not None:
        logger.debug("Checking whether the pattern " + pattern + " is valid for a multi episode")
        valid = valid and validate_name(pattern, multi, anime_type)

    return valid


def check_valid_abd_naming(pattern=None):
    """
    Checks if the name is can be parsed back to its original form for an air-by-date format.

    :return: true if the naming is valid, false if not.
    """
    if pattern is None:
        pattern = settings.NAMING_PATTERN

    logger.debug("Checking whether the pattern " + pattern + " is valid for an air-by-date episode")
    valid = validate_name(pattern, abd=True)

    return valid


def check_valid_sports_naming(pattern=None):
    """
    Checks if the name is can be parsed back to its original form for an sports format.

    :return: true if the naming is valid, false if not.
    """
    if pattern is None:
        pattern = settings.NAMING_PATTERN

    logger.debug("Checking whether the pattern " + pattern + " is valid for an sports episode")
    valid = validate_name(pattern, sports=True)

    return valid


def validate_name(pattern, multi=None, anime_type=None, file_only=False, abd=False, sports=False):
    """
    See if we understand a name

    :param pattern: Name to analyse
    :param multi: Is this a multi-episode name
    :param anime_type: Is this anime
    :param file_only: Is this just a file or a dir
    :param abd: Is air-by-date enabled
    :param sports: Is this sports
    :return: True if valid name, False if not
    """
    ep = generate_sample_ep(multi, abd, sports, anime_type)

    new_name = ep.formatted_filename(pattern, multi, anime_type) + ".ext"
    new_path = ep.formatted_dir(pattern, multi, anime_type)
    if not file_only:
        new_name = os.path.join(new_path, new_name)

    if not new_name:
        logger.debug("Unable to create a name out of " + pattern)
        return False

    logger.debug("Trying to parse " + new_name)

    try:
        result = NameParser(True, showObj=ep.show, naming_pattern=True).parse(new_name)
    except (InvalidNameException, InvalidShowException) as error:
        logger.debug("{0}".format(error))
        return False

    logger.debug("The name " + new_name + " parsed into " + str(result))

    if abd or sports:
        if result.air_date != ep.airdate:
            logger.debug("Air date incorrect in parsed episode, pattern isn't valid")
            return False
    elif anime_type != 3:
        if len(result.ab_episode_numbers) and result.ab_episode_numbers != [x.absolute_number for x in [ep] + ep.relatedEps]:
            logger.debug("Absolute numbering incorrect in parsed episode, pattern isn't valid")
            return False
    else:
        if result.season_number != ep.season:
            logger.debug("Season number incorrect in parsed episode, pattern isn't valid")
            return False
        if result.episode_numbers != [x.episode for x in [ep] + ep.relatedEps]:
            logger.debug("Episode numbering incorrect in parsed episode, pattern isn't valid")
            return False

    return True


def generate_sample_ep(multi=None, abd=False, sports=False, anime_type=None):
    # make a fake episode object
    ep = TVEpisode(2, 3, 3, "Ep Name")

    ep._status = Quality.compositeStatus(DOWNLOADED, Quality.HDTV)
    ep._airdate = datetime.date(2011, 3, 9)

    if abd:
        ep._release_name = "Show.Name.2011.03.09.HDTV.XviD-SICKCHILL"
        ep.show.air_by_date = 1
    elif sports:
        ep._release_name = "Show.Name.2011.03.09.HDTV.XviD-SICKCHILL"
        ep.show.sports = 1
    else:
        if anime_type != 3:
            ep.show.anime = 1
            ep._release_name = "Show.Name.003.HDTV.XviD-SICKCHILL"
        else:
            ep._release_name = "Show.Name.S02E03.HDTV.XviD-SICKCHILL"

    if multi is not None:
        ep._name = "Ep Name (1)"

        if anime_type != 3:
            ep.show.anime = 1

            ep._release_name = "Show.Name.003-004.HDTV.XviD-SICKCHILL"

            secondEp = TVEpisode(2, 4, 4, "Ep Name (2)")
            secondEp._status = Quality.compositeStatus(DOWNLOADED, Quality.HDTV)
            secondEp._release_name = ep._release_name

            ep.relatedEps.append(secondEp)
        else:
            ep._release_name = "Show.Name.S02E03E04E05.HDTV.XviD-SICKCHILL"

            secondEp = TVEpisode(2, 4, 4, "Ep Name (2)")
            secondEp._status = Quality.compositeStatus(DOWNLOADED, Quality.HDTV)
            secondEp._release_name = ep._release_name

            thirdEp = TVEpisode(2, 5, 5, "Ep Name (3)")
            thirdEp._status = Quality.compositeStatus(DOWNLOADED, Quality.HDTV)
            thirdEp._release_name = ep._release_name

            ep.relatedEps.append(secondEp)
            ep.relatedEps.append(thirdEp)

    return ep


def test_name(pattern, multi=None, abd=False, sports=False, anime_type=None):
    ep = generate_sample_ep(multi, abd, sports, anime_type)

    return {"name": ep.formatted_filename(pattern, multi, anime_type), "dir": ep.formatted_dir(pattern, multi, anime_type)}



import requests
import twitter
from requests_oauthlib import OAuth1Session

from sickchill import logger, settings
from sickchill.oldbeard import common


class Notifier(object):
    consumer_key = "vHHtcB6WzpWDG6KYlBMr8g"
    consumer_hash = "zMqq5CB3f8cWKiRO2KzWPTlBanYmV0VYxSXZ0Pxds0E"  # (consumer_secret)

    REQUEST_TOKEN_URL = "https://api.twitter.com/oauth/request_token"
    ACCESS_TOKEN_URL = "https://api.twitter.com/oauth/access_token"
    AUTHORIZATION_URL = "https://api.twitter.com/oauth/authorize"

    def notify_snatch(self, ep_name):
        if settings.TWITTER_NOTIFY_ONSNATCH:
            self._notify_twitter(common.notifyStrings[common.NOTIFY_SNATCH] + ": " + ep_name)

    def notify_download(self, ep_name):
        if settings.TWITTER_NOTIFY_ONDOWNLOAD:
            self._notify_twitter(common.notifyStrings[common.NOTIFY_DOWNLOAD] + ": " + ep_name)

    def notify_subtitle_download(self, ep_name, lang):
        if settings.TWITTER_NOTIFY_ONSUBTITLEDOWNLOAD:
            self._notify_twitter(common.notifyStrings[common.NOTIFY_SUBTITLE_DOWNLOAD] + " " + ep_name + ": " + lang)

    def notify_git_update(self, new_version="??"):
        if settings.USE_TWITTER:
            update_text = common.notifyStrings[common.NOTIFY_GIT_UPDATE_TEXT]
            title = common.notifyStrings[common.NOTIFY_GIT_UPDATE]
            self._notify_twitter(title + " - " + update_text + new_version)

    def notify_login(self, ipaddress=""):
        if settings.USE_TWITTER:
            update_text = common.notifyStrings[common.NOTIFY_LOGIN_TEXT]
            title = common.notifyStrings[common.NOTIFY_LOGIN]
            self._notify_twitter(title + " - " + update_text.format(ipaddress))

    def test_notify(self):
        """
        Tests sending notification.

        :return: True if succeeded, False otherwise
        """
        return self._notify_twitter("This is a test notification from SickChill", force=True)

    def _get_authorization(self):
        """
        Step 1 of authorization - get app authorization url.

        :return: True if succeeded, False otherwise
        """
        logger.debug("Requesting temp token from Twitter")
        oauth_session = OAuth1Session(client_key=self.consumer_key, client_secret=self.consumer_hash)

        try:
            request_token = oauth_session.fetch_request_token(self.REQUEST_TOKEN_URL)
        except requests.exceptions.RequestException as error:
            logger.exception(f"Invalid response from Twitter requesting temp token: {error}")
        else:
            settings.TWITTER_USERNAME = request_token["oauth_token"]
            settings.TWITTER_PASSWORD = request_token["oauth_token_secret"]
            return oauth_session.authorization_url(self.AUTHORIZATION_URL)

    def _get_credentials(self, key):
        logger.info("Type of key is {}".format(type(key)))
        """
        Step 2 of authorization - poll server for access token.

        :param key: Authorization key received from twitter
        :return: True if succeeded, False otherwise
        """
        logger.debug("Generating and signing request for an access token using key " + key)
        oauth_session = OAuth1Session(
            client_key=self.consumer_key,
            client_secret=self.consumer_hash,
            resource_owner_key=settings.TWITTER_USERNAME,
            resource_owner_secret=settings.TWITTER_PASSWORD,
        )

        try:
            access_token = oauth_session.fetch_access_token(self.ACCESS_TOKEN_URL, verifier=str(key))
        except Exception as err:
            logger.exception("The request for a token with did not succeed: {}".format(err))
            return False

        logger.debug("Your Twitter Access Token key: {0}".format(access_token["oauth_token"]))
        logger.debug("Access Token secret: {0}".format(access_token["oauth_token_secret"]))
        settings.TWITTER_USERNAME = access_token["oauth_token"]
        settings.TWITTER_PASSWORD = access_token["oauth_token_secret"]
        return True

    def _send_tweet(self, message=None):
        """
        Sends a tweet.

        :param message: Message to send
        :return: True if succeeded, False otherwise
        """
        api = twitter.Api(
            consumer_key=self.consumer_key,
            consumer_secret=self.consumer_hash,
            access_token_key=settings.TWITTER_USERNAME,
            access_token_secret=settings.TWITTER_PASSWORD,
        )

        logger.debug("Sending tweet: {}".format(message))
        try:
            api.PostUpdate(message[:139])
        except Exception as e:
            logger.exception("Error Sending Tweet: {}".format(str(e)))
            return False

        return True

    def _send_dm(self, message=None):
        """
        Sends a direct message.

        :param message: Message to send
        :return: True if succeeded, False otherwise
        """
        dmdest = settings.TWITTER_DMTO

        api = twitter.Api(
            consumer_key=self.consumer_key,
            consumer_secret=self.consumer_hash,
            access_token_key=settings.TWITTER_USERNAME,
            access_token_secret=settings.TWITTER_PASSWORD,
        )

        logger.debug("Sending DM @{0}: {1}".format(dmdest, message))
        try:
            api.PostDirectMessage(message[:139], screen_name=dmdest)
        except Exception as e:
            logger.exception("Error Sending Tweet (DM): {}".format(str(e)))
            return False

        return True

    def _notify_twitter(self, message="", force=False):
        prefix = settings.TWITTER_PREFIX

        if not settings.USE_TWITTER and not force:
            return False

        if settings.TWITTER_USEDM and settings.TWITTER_DMTO:
            return self._send_dm(prefix + ": " + message)
        else:
            return self._send_tweet(prefix + ": " + message)



# -*- coding: utf-8 -*-
from babelfish import LanguageReverseConverter

from ..exceptions import ConfigurationError


class TheSubDBConverter(LanguageReverseConverter):
    def __init__(self):
        self.from_thesubdb = {'en': ('eng',), 'es': ('spa',), 'fr': ('fra',), 'it': ('ita',), 'nl': ('nld',),
                              'pl': ('pol',), 'pt': ('por', 'BR'), 'ro': ('ron',), 'sv': ('swe',), 'tr': ('tur',)}
        self.to_thesubdb = {v: k for k, v in self.from_thesubdb.items()}
        self.codes = set(self.from_thesubdb.keys())

    def convert(self, alpha3, country=None, script=None):
        if (alpha3, country) in self.to_thesubdb:
            return self.to_thesubdb[(alpha3, country)]
        if (alpha3,) in self.to_thesubdb:
            return self.to_thesubdb[(alpha3,)]

        raise ConfigurationError('Unsupported language for thesubdb: %s, %s, %s' % (alpha3, country, script))

    def reverse(self, thesubdb):
        if thesubdb in self.from_thesubdb:
            return self.from_thesubdb[thesubdb]

        raise ConfigurationError('Unsupported language code for thesubdb: %s' % thesubdb)



"""
Tests for SickChill providers
"""

import unittest

from .test_generic_provider import GenericProviderTests
from .test_nzb_provider import NZBProviderTests
from .test_torrent_provider import TorrentProviderTests

if __name__ == "__main__":
    print('=====> Running all test in "sickchill_tests.providers" <=====')

    TEST_CLASSES = [
        GenericProviderTests,
        NZBProviderTests,
        TorrentProviderTests,
    ]

    for test_class in TEST_CLASSES:
        SUITE = unittest.TestLoader().loadTestsFromTestCase(test_class)
        unittest.TextTestRunner(verbosity=2).run(SUITE)



import threading
import traceback
from queue import Empty, Queue

from .. import logger


class Event(object):
    def __init__(self, event_type):
        self.event_type = event_type

    @property
    def type(self):
        return self.event_type


class Events(threading.Thread):
    def __init__(self, callback):
        super().__init__()
        self.queue = Queue()
        self.daemon = True
        self.callback = callback
        self.name = "EVENT-QUEUE"
        self.stop = threading.Event()

    def put(self, event_type):
        self.queue.put(event_type)

    def run(self):
        """
        Actually runs the thread to process events
        """
        try:
            while not self.stop.is_set():
                try:
                    # get event type
                    event_type = self.queue.get(timeout=1)

                    # perform callback if we got a event type
                    self.callback(event_type)

                    # event completed
                    self.queue.task_done()
                except Empty:
                    pass

            # exiting thread
            self.stop.clear()
        except Exception as e:
            logger.exception("Exception generated in thread " + self.name + ": " + str(e))
            logger.debug(repr(traceback.format_exc()))

    # System Events
    class SystemEvent(Event):
        RESTART = "RESTART"
        SHUTDOWN = "SHUTDOWN"



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class ConnectionPolicyTargetList(ListResource):

    def __init__(self, version, connection_policy_sid):
        """
        Initialize the ConnectionPolicyTargetList

        :param Version version: Version that contains the resource
        :param connection_policy_sid: The SID of the Connection Policy that owns the Target

        :returns: twilio.rest.voice.v1.connection_policy.connection_policy_target.ConnectionPolicyTargetList
        :rtype: twilio.rest.voice.v1.connection_policy.connection_policy_target.ConnectionPolicyTargetList
        """
        super(ConnectionPolicyTargetList, self).__init__(version)

        # Path Solution
        self._solution = {'connection_policy_sid': connection_policy_sid, }
        self._uri = '/ConnectionPolicies/{connection_policy_sid}/Targets'.format(**self._solution)

    def create(self, target, friendly_name=values.unset, priority=values.unset,
               weight=values.unset, enabled=values.unset):
        """
        Create the ConnectionPolicyTargetInstance

        :param unicode target: The SIP address you want Twilio to route your calls to
        :param unicode friendly_name: A string to describe the resource
        :param unicode priority: The relative importance of the target
        :param unicode weight: The value that determines the relative load the Target should receive compared to others with the same priority
        :param bool enabled: Whether the Target is enabled

        :returns: The created ConnectionPolicyTargetInstance
        :rtype: twilio.rest.voice.v1.connection_policy.connection_policy_target.ConnectionPolicyTargetInstance
        """
        data = values.of({
            'Target': target,
            'FriendlyName': friendly_name,
            'Priority': priority,
            'Weight': weight,
            'Enabled': enabled,
        })

        payload = self._version.create(method='POST', uri=self._uri, data=data, )

        return ConnectionPolicyTargetInstance(
            self._version,
            payload,
            connection_policy_sid=self._solution['connection_policy_sid'],
        )

    def stream(self, limit=None, page_size=None):
        """
        Streams ConnectionPolicyTargetInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.voice.v1.connection_policy.connection_policy_target.ConnectionPolicyTargetInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(page_size=limits['page_size'], )

        return self._version.stream(page, limits['limit'])

    def list(self, limit=None, page_size=None):
        """
        Lists ConnectionPolicyTargetInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.voice.v1.connection_policy.connection_policy_target.ConnectionPolicyTargetInstance]
        """
        return list(self.stream(limit=limit, page_size=page_size, ))

    def page(self, page_token=values.unset, page_number=values.unset,
             page_size=values.unset):
        """
        Retrieve a single page of ConnectionPolicyTargetInstance records from the API.
        Request is executed immediately

        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of ConnectionPolicyTargetInstance
        :rtype: twilio.rest.voice.v1.connection_policy.connection_policy_target.ConnectionPolicyTargetPage
        """
        data = values.of({'PageToken': page_token, 'Page': page_number, 'PageSize': page_size, })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return ConnectionPolicyTargetPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of ConnectionPolicyTargetInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of ConnectionPolicyTargetInstance
        :rtype: twilio.rest.voice.v1.connection_policy.connection_policy_target.ConnectionPolicyTargetPage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return ConnectionPolicyTargetPage(self._version, response, self._solution)

    def get(self, sid):
        """
        Constructs a ConnectionPolicyTargetContext

        :param sid: The unique string that identifies the resource

        :returns: twilio.rest.voice.v1.connection_policy.connection_policy_target.ConnectionPolicyTargetContext
        :rtype: twilio.rest.voice.v1.connection_policy.connection_policy_target.ConnectionPolicyTargetContext
        """
        return ConnectionPolicyTargetContext(
            self._version,
            connection_policy_sid=self._solution['connection_policy_sid'],
            sid=sid,
        )

    def __call__(self, sid):
        """
        Constructs a ConnectionPolicyTargetContext

        :param sid: The unique string that identifies the resource

        :returns: twilio.rest.voice.v1.connection_policy.connection_policy_target.ConnectionPolicyTargetContext
        :rtype: twilio.rest.voice.v1.connection_policy.connection_policy_target.ConnectionPolicyTargetContext
        """
        return ConnectionPolicyTargetContext(
            self._version,
            connection_policy_sid=self._solution['connection_policy_sid'],
            sid=sid,
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Voice.V1.ConnectionPolicyTargetList>'


class ConnectionPolicyTargetPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the ConnectionPolicyTargetPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param connection_policy_sid: The SID of the Connection Policy that owns the Target

        :returns: twilio.rest.voice.v1.connection_policy.connection_policy_target.ConnectionPolicyTargetPage
        :rtype: twilio.rest.voice.v1.connection_policy.connection_policy_target.ConnectionPolicyTargetPage
        """
        super(ConnectionPolicyTargetPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of ConnectionPolicyTargetInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.voice.v1.connection_policy.connection_policy_target.ConnectionPolicyTargetInstance
        :rtype: twilio.rest.voice.v1.connection_policy.connection_policy_target.ConnectionPolicyTargetInstance
        """
        return ConnectionPolicyTargetInstance(
            self._version,
            payload,
            connection_policy_sid=self._solution['connection_policy_sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Voice.V1.ConnectionPolicyTargetPage>'


class ConnectionPolicyTargetContext(InstanceContext):

    def __init__(self, version, connection_policy_sid, sid):
        """
        Initialize the ConnectionPolicyTargetContext

        :param Version version: Version that contains the resource
        :param connection_policy_sid: The SID of the Connection Policy that owns the Target
        :param sid: The unique string that identifies the resource

        :returns: twilio.rest.voice.v1.connection_policy.connection_policy_target.ConnectionPolicyTargetContext
        :rtype: twilio.rest.voice.v1.connection_policy.connection_policy_target.ConnectionPolicyTargetContext
        """
        super(ConnectionPolicyTargetContext, self).__init__(version)

        # Path Solution
        self._solution = {'connection_policy_sid': connection_policy_sid, 'sid': sid, }
        self._uri = '/ConnectionPolicies/{connection_policy_sid}/Targets/{sid}'.format(**self._solution)

    def fetch(self):
        """
        Fetch the ConnectionPolicyTargetInstance

        :returns: The fetched ConnectionPolicyTargetInstance
        :rtype: twilio.rest.voice.v1.connection_policy.connection_policy_target.ConnectionPolicyTargetInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return ConnectionPolicyTargetInstance(
            self._version,
            payload,
            connection_policy_sid=self._solution['connection_policy_sid'],
            sid=self._solution['sid'],
        )

    def update(self, friendly_name=values.unset, target=values.unset,
               priority=values.unset, weight=values.unset, enabled=values.unset):
        """
        Update the ConnectionPolicyTargetInstance

        :param unicode friendly_name: A string to describe the resource
        :param unicode target: The SIP address you want Twilio to route your calls to
        :param unicode priority: The relative importance of the target
        :param unicode weight: The value that determines the relative load the Target should receive compared to others with the same priority
        :param bool enabled: Whether the Target is enabled

        :returns: The updated ConnectionPolicyTargetInstance
        :rtype: twilio.rest.voice.v1.connection_policy.connection_policy_target.ConnectionPolicyTargetInstance
        """
        data = values.of({
            'FriendlyName': friendly_name,
            'Target': target,
            'Priority': priority,
            'Weight': weight,
            'Enabled': enabled,
        })

        payload = self._version.update(method='POST', uri=self._uri, data=data, )

        return ConnectionPolicyTargetInstance(
            self._version,
            payload,
            connection_policy_sid=self._solution['connection_policy_sid'],
            sid=self._solution['sid'],
        )

    def delete(self):
        """
        Deletes the ConnectionPolicyTargetInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._version.delete(method='DELETE', uri=self._uri, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Voice.V1.ConnectionPolicyTargetContext {}>'.format(context)


class ConnectionPolicyTargetInstance(InstanceResource):

    def __init__(self, version, payload, connection_policy_sid, sid=None):
        """
        Initialize the ConnectionPolicyTargetInstance

        :returns: twilio.rest.voice.v1.connection_policy.connection_policy_target.ConnectionPolicyTargetInstance
        :rtype: twilio.rest.voice.v1.connection_policy.connection_policy_target.ConnectionPolicyTargetInstance
        """
        super(ConnectionPolicyTargetInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'account_sid': payload.get('account_sid'),
            'connection_policy_sid': payload.get('connection_policy_sid'),
            'sid': payload.get('sid'),
            'friendly_name': payload.get('friendly_name'),
            'target': payload.get('target'),
            'priority': deserialize.integer(payload.get('priority')),
            'weight': deserialize.integer(payload.get('weight')),
            'enabled': payload.get('enabled'),
            'date_created': deserialize.iso8601_datetime(payload.get('date_created')),
            'date_updated': deserialize.iso8601_datetime(payload.get('date_updated')),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {
            'connection_policy_sid': connection_policy_sid,
            'sid': sid or self._properties['sid'],
        }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: ConnectionPolicyTargetContext for this ConnectionPolicyTargetInstance
        :rtype: twilio.rest.voice.v1.connection_policy.connection_policy_target.ConnectionPolicyTargetContext
        """
        if self._context is None:
            self._context = ConnectionPolicyTargetContext(
                self._version,
                connection_policy_sid=self._solution['connection_policy_sid'],
                sid=self._solution['sid'],
            )
        return self._context

    @property
    def account_sid(self):
        """
        :returns: The SID of the Account that created the resource
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def connection_policy_sid(self):
        """
        :returns: The SID of the Connection Policy that owns the Target
        :rtype: unicode
        """
        return self._properties['connection_policy_sid']

    @property
    def sid(self):
        """
        :returns: The unique string that identifies the resource
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def friendly_name(self):
        """
        :returns: The string that you assigned to describe the resource
        :rtype: unicode
        """
        return self._properties['friendly_name']

    @property
    def target(self):
        """
        :returns: The SIP address you want Twilio to route your calls to
        :rtype: unicode
        """
        return self._properties['target']

    @property
    def priority(self):
        """
        :returns: The relative importance of the target
        :rtype: unicode
        """
        return self._properties['priority']

    @property
    def weight(self):
        """
        :returns: The value that determines the relative load the Target should receive compared to others with the same priority
        :rtype: unicode
        """
        return self._properties['weight']

    @property
    def enabled(self):
        """
        :returns: Whether the target is enabled
        :rtype: bool
        """
        return self._properties['enabled']

    @property
    def date_created(self):
        """
        :returns: The RFC 2822 date and time in GMT when the resource was created
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def date_updated(self):
        """
        :returns: The RFC 2822 date and time in GMT when the resource was last updated
        :rtype: datetime
        """
        return self._properties['date_updated']

    @property
    def url(self):
        """
        :returns: The absolute URL of the resource
        :rtype: unicode
        """
        return self._properties['url']

    def fetch(self):
        """
        Fetch the ConnectionPolicyTargetInstance

        :returns: The fetched ConnectionPolicyTargetInstance
        :rtype: twilio.rest.voice.v1.connection_policy.connection_policy_target.ConnectionPolicyTargetInstance
        """
        return self._proxy.fetch()

    def update(self, friendly_name=values.unset, target=values.unset,
               priority=values.unset, weight=values.unset, enabled=values.unset):
        """
        Update the ConnectionPolicyTargetInstance

        :param unicode friendly_name: A string to describe the resource
        :param unicode target: The SIP address you want Twilio to route your calls to
        :param unicode priority: The relative importance of the target
        :param unicode weight: The value that determines the relative load the Target should receive compared to others with the same priority
        :param bool enabled: Whether the Target is enabled

        :returns: The updated ConnectionPolicyTargetInstance
        :rtype: twilio.rest.voice.v1.connection_policy.connection_policy_target.ConnectionPolicyTargetInstance
        """
        return self._proxy.update(
            friendly_name=friendly_name,
            target=target,
            priority=priority,
            weight=weight,
            enabled=enabled,
        )

    def delete(self):
        """
        Deletes the ConnectionPolicyTargetInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._proxy.delete()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Voice.V1.ConnectionPolicyTargetInstance {}>'.format(context)



import datetime
import os
import traceback

import sickchill
from sickchill import logger, settings
from sickchill.helper.common import episode_num, sanitize_filename
from sickchill.oldbeard.trakt_api import TraktAPI, traktException
from sickchill.show.Show import Show

from . import db, helpers, search_queue
from .common import Quality, SKIPPED, UNKNOWN, WANTED


def setEpisodeToWanted(show, s, e):
    """
    Sets an episode to wanted, only if it is currently skipped
    """
    epObj = show.getEpisode(s, e)
    if epObj:

        with epObj.lock:
            if epObj.status != SKIPPED or epObj.airdate == datetime.date.min:
                return

            logger.info("Setting episode {show} {ep} to wanted".format(show=show.name, ep=episode_num(s, e)))
            # figure out what segment the episode is in and remember it so we can backlog it

            epObj.status = WANTED
            epObj.saveToDB()

        cur_backlog_queue_item = search_queue.BacklogQueueItem(show, [epObj])
        settings.searchQueueScheduler.action.add_item(cur_backlog_queue_item)

        logger.info("Starting backlog search for {show} {ep} because some episodes were set to wanted".format(show=show.name, ep=episode_num(s, e)))


class TraktChecker(object):
    def __init__(self):
        self.trakt_api = TraktAPI(settings.SSL_VERIFY, settings.TRAKT_TIMEOUT)
        self.todoBacklog = []
        self.todoWanted = []
        self.ShowWatchlist = {}
        self.EpisodeWatchlist = {}
        self.Collectionlist = {}
        self.amActive = False

    def run(self, force=False):
        self.amActive = True

        # add shows from trakt.tv watchlist
        if settings.TRAKT_SYNC_WATCHLIST:
            self.todoWanted = []  # its about to all get re-added
            if len(settings.ROOT_DIRS.split("|")) < 2:
                logger.warning("No default root directory")
                return

            try:
                self.syncWatchlist()
            except Exception:
                logger.debug(traceback.format_exc())

            try:
                # sync trakt.tv library with sickchill library
                self.syncLibrary()
            except Exception:
                logger.debug(traceback.format_exc())

        self.amActive = False

    def findShow(self, indexer, indexerid):
        traktShow = None

        try:
            library = self.trakt_api.traktRequest("sync/collection/shows") or []
            if not library:
                logger.debug("No shows found in your library, aborting library update")
                return

            traktShow = [x for x in library if int(indexerid) == int(x["show"]["ids"][sickchill.indexer.slug(indexer)] or -1)]
        except traktException as e:
            logger.warning("Could not connect to Trakt service. Aborting library check. Error: {0}".format(repr(e)))

        return traktShow

    def removeShowFromTraktLibrary(self, show_obj):
        if self.findShow(show_obj.indexer, show_obj.indexerid):

            # URL parameters
            data = {"shows": [{"title": show_obj.name, "year": show_obj.startyear, "ids": {show_obj.idxr.slug: show_obj.indexerid}}]}

            logger.debug("Removing {0} from trakt.tv library".format(show_obj.name))

            try:
                self.trakt_api.traktRequest("sync/collection/remove", data, method="POST")
            except traktException as e:
                logger.warning("Could not connect to Trakt service. Aborting removing show {0} from Trakt library. Error: {1}".format(show_obj.name, repr(e)))

    def addShowToTraktLibrary(self, show_obj):
        """
        Sends a request to trakt indicating that the given show and all its episodes is part of our library.

        show_obj: The TVShow object to add to trakt
        """
        if not self.findShow(show_obj.indexer, show_obj.indexerid):
            # URL parameters
            data = {"shows": [{"title": show_obj.name, "year": show_obj.startyear, "ids": {show_obj.idxr.slug: show_obj.indexerid}}]}

            logger.debug("Adding {0} to trakt.tv library".format(show_obj.name))

            try:
                self.trakt_api.traktRequest("sync/collection", data, method="POST")
            except traktException as e:
                logger.warning("Could not connect to Trakt service. Aborting adding show {0} to Trakt library. Error: {1}".format(show_obj.name, repr(e)))
                return

    def syncLibrary(self):
        if settings.TRAKT_SYNC and settings.USE_TRAKT:
            logger.debug("Sync SickChill with Trakt Collection")

            if self._getShowCollection():
                self.addEpisodeToTraktCollection()
                if settings.TRAKT_SYNC_REMOVE:
                    self.removeEpisodeFromTraktCollection()

    def removeEpisodeFromTraktCollection(self):
        if settings.TRAKT_SYNC_REMOVE and settings.TRAKT_SYNC and settings.USE_TRAKT:
            logger.debug("COLLECTION::REMOVE::START - Look for Episodes to Remove From Trakt Collection")

            main_db_con = db.DBConnection()
            sql_selection = "select tv_shows.indexer, tv_shows.startyear, showid, show_name, season, episode, tv_episodes.status, tv_episodes.location from tv_episodes,tv_shows where tv_shows.indexer_id = tv_episodes.showid"
            episodes = main_db_con.select(sql_selection)

            if episodes is not None:
                trakt_data = []

                for cur_episode in episodes:
                    if self._checkInList(
                        sickchill.indexer.slug(cur_episode["indexer"]),
                        str(cur_episode["showid"]),
                        str(cur_episode["season"]),
                        str(cur_episode["episode"]),
                        List="Collection",
                    ):
                        if cur_episode["location"] == "":
                            logger.debug(
                                "Removing Episode {show} {ep} from collection".format(
                                    show=cur_episode["show_name"], ep=episode_num(cur_episode["season"], cur_episode["episode"])
                                )
                            )
                            trakt_data.append(
                                (
                                    cur_episode["showid"],
                                    cur_episode["indexer"],
                                    cur_episode["show_name"],
                                    cur_episode["startyear"],
                                    cur_episode["season"],
                                    cur_episode["episode"],
                                )
                            )

                if trakt_data:
                    try:
                        data = self.trakt_bulk_data_generate(trakt_data)
                        self.trakt_api.traktRequest("sync/collection/remove", data, method="POST")
                        self._getShowCollection()
                    except traktException as e:
                        logger.warning("Could not connect to Trakt service. Error: {0}".format(str(e)))

            logger.debug("COLLECTION::REMOVE::FINISH - Look for Episodes to Remove From Trakt Collection")

    def addEpisodeToTraktCollection(self):
        if settings.TRAKT_SYNC and settings.USE_TRAKT:
            logger.debug("COLLECTION::ADD::START - Look for Episodes to Add to Trakt Collection")

            main_db_con = db.DBConnection()
            sql_selection = (
                "select tv_shows.indexer, tv_shows.startyear, showid, show_name, season, episode from tv_episodes,tv_shows where tv_shows.indexer_id = tv_episodes.showid and tv_episodes.status in ("
                + ",".join([str(x) for x in Quality.DOWNLOADED + Quality.ARCHIVED])
                + ")"
            )
            episodes = main_db_con.select(sql_selection)

            if episodes is not None:
                trakt_data = []

                for cur_episode in episodes:
                    if not self._checkInList(
                        sickchill.indexer.slug(cur_episode["indexer"]),
                        str(cur_episode["showid"]),
                        str(cur_episode["season"]),
                        str(cur_episode["episode"]),
                        List="Collection",
                    ):
                        logger.debug(
                            "Adding Episode {show} {ep} to collection".format(
                                show=cur_episode["show_name"], ep=episode_num(cur_episode["season"], cur_episode["episode"])
                            )
                        )
                        trakt_data.append(
                            (
                                cur_episode["showid"],
                                cur_episode["indexer"],
                                cur_episode["show_name"],
                                cur_episode["startyear"],
                                cur_episode["season"],
                                cur_episode["episode"],
                            )
                        )

                if trakt_data:
                    try:
                        data = self.trakt_bulk_data_generate(trakt_data)
                        self.trakt_api.traktRequest("sync/collection", data, method="POST")
                        self._getShowCollection()
                    except traktException as e:
                        logger.warning("Could not connect to Trakt service. Error: {0}".format(str(e)))

            logger.debug("COLLECTION::ADD::FINISH - Look for Episodes to Add to Trakt Collection")

    def syncWatchlist(self):
        if settings.TRAKT_SYNC_WATCHLIST and settings.USE_TRAKT:
            logger.debug("Sync SickChill with Trakt Watchlist")

            self.removeShowFromSickChill()

            if self._getShowWatchlist():
                self.addShowToTraktWatchList()
                self.updateShows()

            if self._getEpisodeWatchlist():
                self.removeEpisodeFromTraktWatchList()
                self.addEpisodeToTraktWatchList()
                self.updateEpisodes()

    def removeEpisodeFromTraktWatchList(self):
        if settings.TRAKT_SYNC_WATCHLIST and settings.USE_TRAKT:
            logger.debug("WATCHLIST::REMOVE::START - Look for Episodes to Remove from Trakt Watchlist")

            main_db_con = db.DBConnection()
            sql_selection = "select tv_shows.indexer, tv_shows.startyear, showid, show_name, season, episode, tv_episodes.status from tv_episodes,tv_shows where tv_shows.indexer_id = tv_episodes.showid"
            episodes = main_db_con.select(sql_selection)

            if episodes is not None:
                trakt_data = []

                for cur_episode in episodes:
                    if self._checkInList(
                        sickchill.indexer.slug(cur_episode["indexer"]), str(cur_episode["showid"]), str(cur_episode["season"]), str(cur_episode["episode"])
                    ):
                        if cur_episode["status"] not in Quality.SNATCHED + Quality.SNATCHED_PROPER + [UNKNOWN] + [WANTED]:
                            logger.debug(
                                "Removing Episode {show} {ep} from watchlist".format(
                                    show=cur_episode["show_name"], ep=episode_num(cur_episode["season"], cur_episode["episode"])
                                )
                            )
                            trakt_data.append(
                                (
                                    cur_episode["showid"],
                                    cur_episode["indexer"],
                                    cur_episode["show_name"],
                                    cur_episode["startyear"],
                                    cur_episode["season"],
                                    cur_episode["episode"],
                                )
                            )

                if trakt_data:
                    try:
                        data = self.trakt_bulk_data_generate(trakt_data)
                        self.trakt_api.traktRequest("sync/watchlist/remove", data, method="POST")
                        self._getEpisodeWatchlist()
                    except traktException as e:
                        logger.warning("Could not connect to Trakt service. Error: {0}".format(str(e)))

                logger.debug("WATCHLIST::REMOVE::FINISH - Look for Episodes to Remove from Trakt Watchlist")

    def addEpisodeToTraktWatchList(self):
        if settings.TRAKT_SYNC_WATCHLIST and settings.USE_TRAKT:
            logger.debug("WATCHLIST::ADD::START - Look for Episodes to Add to Trakt Watchlist")

            main_db_con = db.DBConnection()
            sql_selection = (
                "select tv_shows.indexer, tv_shows.startyear, showid, show_name, season, episode from tv_episodes,tv_shows where tv_shows.indexer_id = tv_episodes.showid and tv_episodes.status in ("
                + ",".join([str(x) for x in Quality.SNATCHED + Quality.SNATCHED_PROPER + [WANTED]])
                + ")"
            )
            episodes = main_db_con.select(sql_selection)

            if episodes is not None:
                trakt_data = []

                for cur_episode in episodes:
                    if not self._checkInList(
                        sickchill.indexer.slug(cur_episode["indexer"]), str(cur_episode["showid"]), str(cur_episode["season"]), str(cur_episode["episode"])
                    ):
                        logger.debug(
                            "Adding Episode {show} {ep} to watchlist".format(
                                show=cur_episode["show_name"], ep=episode_num(cur_episode["season"], cur_episode["episode"])
                            )
                        )
                        trakt_data.append(
                            (
                                cur_episode["showid"],
                                cur_episode["indexer"],
                                cur_episode["show_name"],
                                cur_episode["startyear"],
                                cur_episode["season"],
                                cur_episode["episode"],
                            )
                        )

                if trakt_data:
                    try:
                        data = self.trakt_bulk_data_generate(trakt_data)
                        self.trakt_api.traktRequest("sync/watchlist", data, method="POST")
                        self._getEpisodeWatchlist()
                    except traktException as e:
                        logger.warning("Could not connect to Trakt service. Error {0}".format(str(e)))

            logger.debug("WATCHLIST::ADD::FINISH - Look for Episodes to Add to Trakt Watchlist")

    def addShowToTraktWatchList(self):
        if settings.TRAKT_SYNC_WATCHLIST and settings.USE_TRAKT:
            logger.debug("SHOW_WATCHLIST::ADD::START - Look for Shows to Add to Trakt Watchlist")

            if settings.showList is not None:
                trakt_data = []

                for show in settings.showList:
                    if not self._checkInList(show.idxr.slug, str(show.indexerid), "0", "0", List="Show"):
                        logger.debug("Adding Show: Indexer {0} {1} - {2} to Watchlist".format(show.idxr.name, str(show.indexerid), show.name))
                        show_el = {"title": show.name, "year": show.startyear, "ids": {show.idxr.slug: show.indexerid}}

                        trakt_data.append(show_el)

                if trakt_data:
                    try:
                        data = {"shows": trakt_data}
                        self.trakt_api.traktRequest("sync/watchlist", data, method="POST")
                        self._getShowWatchlist()
                    except traktException as e:
                        logger.warning("Could not connect to Trakt service. Error: {0}".format(str(e)))

            logger.debug("SHOW_WATCHLIST::ADD::FINISH - Look for Shows to Add to Trakt Watchlist")

    def removeShowFromSickChill(self):
        if settings.TRAKT_SYNC_WATCHLIST and settings.USE_TRAKT and settings.TRAKT_REMOVE_SHOW_FROM_SICKCHILL:
            logger.debug("SHOW_SICKCHILL::REMOVE::START - Look for Shows to remove from SickChill")

            if settings.showList:
                for show in settings.showList:
                    if show.status in ("Ended", "Canceled"):
                        if not show.imdb_id:
                            logger.warning(
                                "Could not check trakt progress for {0} because the imdb id is missing from {} data, skipping".format(show.name, show.idxr.name)
                            )
                            continue

                        try:
                            progress = self.trakt_api.traktRequest("shows/" + show.imdb_id + "/progress/watched") or {}
                        except traktException as e:
                            logger.warning(
                                "Could not connect to Trakt service. Aborting removing show {0} from SickChill. Error: {1}".format(show.name, repr(e))
                            )
                            continue

                        if not progress:
                            continue

                        if progress.get("aired", True) == progress.get("completed", False):
                            settings.showQueueScheduler.action.remove_show(show, full=True)
                            logger.debug("Show: {0} has been removed from SickChill".format(show.name))

            logger.debug("SHOW_SICKCHILL::REMOVE::FINISH - Trakt Show Watchlist")

    def updateShows(self):
        logger.debug("SHOW_WATCHLIST::CHECK::START - Trakt Show Watchlist")

        self._getShowWatchlist()
        if not self.ShowWatchlist:
            logger.debug("No shows found in your watchlist, aborting watchlist update")
            return

        for index, indexer in sickchill.indexer:
            if indexer.slug in self.ShowWatchlist:
                for show_el in self.ShowWatchlist[indexer.slug]:
                    indexer_id = int(str(show_el))
                    show = self.ShowWatchlist[indexer.slug][show_el]

                    # logger.debug("Checking Show: %s %s %s" % (slug, indexer_id, show['title']))
                    if int(settings.TRAKT_METHOD_ADD) != 2:
                        self.addDefaultShow(index, indexer_id, show["title"], SKIPPED)
                    else:
                        self.addDefaultShow(index, indexer_id, show["title"], WANTED)

                    if int(settings.TRAKT_METHOD_ADD) == 1:
                        newShow = Show.find(settings.showList, indexer_id)

                        if newShow is not None:
                            setEpisodeToWanted(newShow, 1, 1)
                        else:
                            self.todoWanted.append((indexer_id, 1, 1))
        logger.debug("SHOW_WATCHLIST::CHECK::FINISH - Trakt Show Watchlist")

    def updateEpisodes(self):
        """
        Sets episodes to wanted that are in trakt watchlist
        """
        logger.debug("SHOW_WATCHLIST::CHECK::START - Trakt Episode Watchlist")

        self._getEpisodeWatchlist()
        if not self.EpisodeWatchlist:
            logger.debug("No episode found in your watchlist, aborting episode update")
            return

        managed_show = []

        for index, indexer in sickchill.indexer:
            if indexer.slug in self.EpisodeWatchlist:
                for show_el in self.EpisodeWatchlist[indexer.slug]:
                    indexer_id = int(show_el)
                    show = self.EpisodeWatchlist[indexer.slug][show_el]

                    newShow = Show.find(settings.showList, indexer_id)

                    try:
                        if newShow is None:
                            if indexer_id not in managed_show:
                                self.addDefaultShow(index, indexer_id, show["title"], SKIPPED)
                                managed_show.append(indexer_id)

                                for season_el in show["seasons"]:
                                    season = int(season_el)

                                    for episode_el in show["seasons"][season_el]["episodes"]:
                                        self.todoWanted.append((indexer_id, season, int(episode_el)))
                        else:
                            if newShow.indexer == indexer:
                                for season_el in show["seasons"]:
                                    season = int(season_el)

                                    for episode_el in show["seasons"][season_el]["episodes"]:
                                        setEpisodeToWanted(newShow, season, int(episode_el))
                    except TypeError:
                        logger.debug("Could not parse the output from trakt for {0} ".format(show["title"]))
        logger.debug("SHOW_WATCHLIST::CHECK::FINISH - Trakt Episode Watchlist")

    @staticmethod
    def addDefaultShow(indexer, indexer_id, name, status):
        """
        Adds a new show with the default settings
        """
        if not Show.find(settings.showList, int(indexer_id)):
            logger.info("Adding show " + str(indexer_id))
            root_dirs = settings.ROOT_DIRS.split("|")

            try:
                location = root_dirs[int(root_dirs[0]) + 1]
            except Exception:
                location = None

            if location:
                showPath = os.path.join(location, sanitize_filename(name))
                dir_exists = helpers.makeDir(showPath)

                if not dir_exists:
                    logger.warning("Unable to create the folder {0} , can't add the show".format(showPath))
                    return
                else:
                    helpers.chmodAsParent(showPath)

                settings.showQueueScheduler.action.add_show(
                    int(indexer),
                    int(indexer_id),
                    showPath,
                    default_status=status,
                    quality=int(settings.QUALITY_DEFAULT),
                    season_folders=int(settings.SEASON_FOLDERS_DEFAULT),
                    paused=settings.TRAKT_START_PAUSED,
                    default_status_after=status,
                )
            else:
                logger.warning("There was an error creating the show, no root directory setting found")
                return

    def manageNewShow(self, show):
        logger.debug("Checking if trakt watch list wants to search for episodes from new show " + show.name)
        episodes = [i for i in self.todoWanted if i[0] == show.indexerid]

        for episode in episodes:
            self.todoWanted.remove(episode)
            setEpisodeToWanted(show, episode[1], episode[2])

    def _checkInList(self, slug, showid, season, episode, List=None):
        """
        Check in the Watchlist or CollectionList for Show
        Is the Show, Season and Episode in the slug list
        """
        # logger.debug("Checking Show: %s %s %s " % (slug, showid, List))

        if "Collection" == List:
            try:
                if self.Collectionlist[slug][showid]["seasons"][season]["episodes"][episode] == episode:
                    return True
            except Exception:
                return False
        elif "Show" == List:
            try:
                if self.ShowWatchlist[slug][showid]["id"] == showid:
                    return True
            except Exception:
                return False
        else:
            try:
                if self.EpisodeWatchlist[slug][showid]["seasons"][season]["episodes"][episode] == episode:
                    return True
            except Exception:
                return False

    def _getShowWatchlist(self):
        """
        Get Watchlist and parse once into addressable structure
        """
        try:
            self.ShowWatchlist = {indexer.slug: {} for index, indexer in sickchill.indexer}
            TraktShowWatchlist = self.trakt_api.traktRequest("sync/watchlist/shows")
            for slug in self.ShowWatchlist:
                for watchlist_el in TraktShowWatchlist:
                    if watchlist_el["show"]["ids"][slug]:

                        title = watchlist_el["show"]["title"]
                        year = str(watchlist_el["show"]["year"])

                        showid = str(watchlist_el["show"]["ids"][slug])
                        self.ShowWatchlist[slug][showid] = {"id": showid, "title": title, "year": year}
        except traktException as e:
            logger.warning("Could not connect to trakt service, cannot download Show Watchlist: {0}".format(repr(e)))
            return False
        return True

    def _getEpisodeWatchlist(self):
        """
        Get Watchlist and parse once into addressable structure
        """
        try:
            self.EpisodeWatchlist = {indexer.slug: {} for index, indexer in sickchill.indexer}
            TraktEpisodeWatchlist = self.trakt_api.traktRequest("sync/watchlist/episodes")
            for slug in self.EpisodeWatchlist:
                for watchlist_el in TraktEpisodeWatchlist:
                    if watchlist_el["show"]["ids"][slug]:

                        title = watchlist_el["show"]["title"]
                        year = str(watchlist_el["show"]["year"])
                        season = str(watchlist_el["episode"]["season"])
                        episode = str(watchlist_el["episode"]["number"])

                        showid = str(watchlist_el["show"]["ids"][slug])

                        if showid not in self.EpisodeWatchlist[slug]:
                            self.EpisodeWatchlist[slug][showid] = {"id": showid, "title": title, "year": year, "seasons": {}}

                        if season not in self.EpisodeWatchlist[slug][showid]["seasons"]:
                            self.EpisodeWatchlist[slug][showid]["seasons"][season] = {"s": season, "episodes": {}}

                        if episode not in self.EpisodeWatchlist[slug][showid]["seasons"][season]["episodes"]:
                            self.EpisodeWatchlist[slug][showid]["seasons"][season]["episodes"][episode] = episode

        except traktException as e:
            logger.warning("Could not connect to trakt service, cannot download Episode Watchlist: {0}".format(repr(e)))
            return False
        return True

    def _getShowCollection(self):
        """
        Get Collection and parse once into addressable structure
        """
        try:
            self.Collectionlist = {indexer.slug: {} for index, indexer in sickchill.indexer}
            logger.debug("Getting Show Collection")
            TraktCollectionList = self.trakt_api.traktRequest("sync/collection/shows")
            for slug in self.Collectionlist:
                for watchlist_el in TraktCollectionList:
                    if "seasons" in watchlist_el:
                        for season_el in watchlist_el["seasons"]:
                            for episode_el in season_el["episodes"]:
                                season = str(season_el["number"])
                                episode = str(episode_el["number"])

                                if watchlist_el["show"]["ids"][slug] is not None:
                                    title = watchlist_el["show"]["title"]
                                    year = str(watchlist_el["show"]["year"])
                                    showid = str(watchlist_el["show"]["ids"][slug])

                                    if showid not in self.Collectionlist[slug]:
                                        self.Collectionlist[slug][showid] = {"id": showid, "title": title, "year": year, "seasons": {}}

                                    if season not in self.Collectionlist[slug][showid]["seasons"]:
                                        self.Collectionlist[slug][showid]["seasons"][season] = {"s": season, "episodes": {}}

                                    if episode not in self.Collectionlist[slug][showid]["seasons"][season]["episodes"]:
                                        self.Collectionlist[slug][showid]["seasons"][season]["episodes"][episode] = episode
        except traktException as e:
            logger.warning("Could not connect to trakt service, cannot download Show Collection: {0}".format(repr(e)))
            return False
        return True

    @staticmethod
    def trakt_bulk_data_generate(data):
        """
        Build the JSON structure to send back to Trakt
        """
        uniqueShows = {}
        uniqueSeasons = {}

        for showid, indexer, show_name, startyear, season, episode in data:
            slug = sickchill.indexer.slug(indexer)
            if showid not in uniqueShows:
                uniqueShows[showid] = {"title": show_name, "year": startyear, "ids": {slug: showid}, "seasons": []}
                uniqueSeasons[showid] = []

            if season not in uniqueSeasons[showid]:
                uniqueSeasons[showid].append(season)

        # build the query
        showList = []
        seasonsList = {}

        for searchedShow in uniqueShows:
            seasonsList[searchedShow] = []

            for searchedSeason in uniqueSeasons[searchedShow]:
                episodesList = []

                for showid, indexerid, show_name, startyear, season, episode in data:
                    if season == searchedSeason and showid == searchedShow:
                        episodesList.append({"number": episode})
                uniqueShows[searchedShow]["seasons"].append({"number": searchedSeason, "episodes": episodesList})
            showList.append(uniqueShows[searchedShow])
        post_data = {"shows": showList}
        return post_data
